/* 
 * File:   main.cpp
 * Author: Sadaji Sunaga
 *
 * Created on 2016/11/21, 16:23
 */

#include "rpI2c.h"
#include "bme280.h"
#include <gtkmm.h>
//#include <gdatetime.h> 
//#include <direct.h>
//#include <sys\stat.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

class MyDrawArea : public Gtk::DrawingArea
{
    CI2C m_i2c;
    CBme280 m_bme280;
public:

    MyDrawArea()
    {
        Glib::signal_timeout().connect(sigc::mem_fun(*this, &MyDrawArea::timer_1s), 1000);
        m_bme280.init(&m_i2c);
    }

protected:
    bool timer_1s();
    virtual bool on_expose_event(GdkEventExpose* e);
    void WriteCsv(gint h, gint m, gint s, double temp, double humidi, double press);
};

bool MyDrawArea::timer_1s()
{
    Glib::RefPtr<Gdk::Window> win = get_window();
    if (win)
    {
        Gdk::Rectangle r(0, 0, get_allocation().get_width(),
                get_allocation().get_height());
        win->invalidate_rect(r, false);
    }
//    int32_t br = m_i2c.GetBaudrate();
//    m_i2c.SetBaudrate(50000);
    return true;
}

void MyDrawArea::WriteCsv(gint h, gint m, gint s, double temp, double humidi, double press )
{
   gint yyyy, mm, dd,len;
   char szFileName[128];
   char szPathName[128];
   char buf[1024];
   
   memset(szFileName, 0, 128);
   memset(szPathName, 0, 128);
   
    GDateTime *gdt = g_date_time_new_now_local();
    yyyy = g_date_time_get_year(gdt);
    mm = g_date_time_get_month(gdt);
    dd =  g_date_time_get_day_of_month(gdt);
    strcpy(szPathName, "/home/pi/weather");
    int a = mkdir(szPathName,
                S_IRUSR | S_IWUSR | S_IXUSR |         /* rwx */
                S_IRGRP | S_IWGRP | S_IXGRP |         /* rwx */
                S_IROTH | S_IXOTH | S_IXOTH    )   ;
    sprintf(szFileName, "/%4d%02d%02d.csv", yyyy, mm, dd);
    strcat(szPathName, szFileName);
    int fd = open(szPathName, O_RDWR |O_APPEND); 
    if(fd == -1)
    {
        fd = open(szPathName, O_CREAT|O_RDWR |O_APPEND); 
        if(fd == -1)
        {
            return;
        }
        chmod(szPathName,
               S_IRUSR | S_IWUSR | S_IXUSR |    /* rwx */
                S_IRGRP | S_IXGRP |              /* r-x */
                S_IROTH | S_IXOTH );
        strcpy(buf, "時刻,温度,湿度,気圧\r\n");
        write(fd, buf, len);
    }
    lseek(fd, SEEK_END, 0);
    sprintf(buf, "%d:%02d:%02d,%.1f,%.1f,%.1f\r\n",h, m ,s, temp, humidi, press);
    len = strlen(buf);
    write(fd, buf, len);
   close(fd);
}

bool MyDrawArea::on_expose_event(GdkEventExpose* event)
{
//    double data;
    double temp, humidi, press;
    int x = 0;
    int width = get_width();
    int height = get_height();
    double y = 10;
 
    Cairo::RefPtr< Cairo::Context > cairoctx = get_window()->create_cairo_context();

    // 解像度取得
    double resolution = get_pango_context()->get_resolution();

    // 背景の塗りつぶし
    Gdk::Cairo::set_source_color(cairoctx, Gdk::Color("white"));
    cairoctx->paint();

    //  現在時刻の取得
    GDateTime *gdt = g_date_time_new_now_local();
    gint h = g_date_time_get_hour(gdt),
            m = g_date_time_get_minute(gdt),
            s = g_date_time_get_second(gdt);
//    g_date_time_get_ymd(gdt, &yyyy, &mm. &dd);
 //           ymd = g_date_time_get_ymd(gdt);
    //  文字列 "12:34.56" に変換
    gchar buf[100];
    sprintf(buf, "%2d:%02d:%02d", h, m, s);
    // フォント指定
    cairoctx->select_font_face("Monospace", Cairo::FONT_SLANT_NORMAL, Cairo::FONT_WEIGHT_NORMAL);
    cairoctx->set_font_size(24 * resolution / 72);
    // 色指定
    Gdk::Cairo::set_source_color(cairoctx, Gdk::Color("black"));

    Glib::ustring text;
    //時刻により文字幅が変わるので固定にする
    text = "時刻： 99:99:99";
    Cairo::TextExtents txext;
    cairoctx->get_text_extents(text, txext);
    x = width / 2 - txext.width / 2 - txext.x_bearing;

    //show  time 
    text = "時刻： ";
    text.append(buf, strlen(buf));

    // 描画(センタリング)
    cairoctx->move_to(x, y - txext.y_bearing);
    cairoctx->show_text(text);

    y += txext.height + 10;
    text = "温度： ";
    temp = m_bme280.getTemparature();
    sprintf(buf, "%.1f ", temp);
    text.append(buf, strlen(buf));
    text.append("℃");
    cairoctx->move_to(x, y - txext.y_bearing);
    cairoctx->show_text(text);

    // 次の文字列のy座標
    y += txext.height + 10;

    text = "湿度： ";
    humidi = m_bme280.getHumidity();
    sprintf(buf, "%.1f  %%", humidi);
    text.append(buf, strlen(buf));

    cairoctx->move_to(x, y - txext.y_bearing);
    cairoctx->show_text(text);

    // 次の文字列のy座標
    y += txext.height + 10;

    text = "気圧： ";
    press = m_bme280.getPressure();
    sprintf(buf, "%.1f hPa", press);
    text.append(buf, strlen(buf));

    cairoctx->move_to(x, y - txext.y_bearing);
    cairoctx->show_text(text);

  if( (s % 60) == 0 )
    {
        WriteCsv(h, m, s, temp, humidi, press);
    }
      
    return true;
}

class MainWin : public Gtk::Window
{
    MyDrawArea m_drawarea;
    bool timer_1s();

public:
    //constructor
    MainWin()
    {
        add(m_drawarea);
        show_all_children();
        resize(360, 180);
        set_title("気象データの観測");
    }
};

int main(int argc, char *argv[])
{
    Gtk::Main kit(argc, argv);
    MainWin mainwin;
    Gtk::Main::run(mainwin);
    return 0;
}

